def reduct(model, attributes, dec) :
    k = model.konsistensi_tabel(attrs, dec)

    remove_attributes = [] 
    for a in attributes:
        R = remove_attributes.copy()
        R.append(a) 
        S = set(attributes) - set(R)
        k_s = model.konsistensi_tabel(list(S), dec)
        if k_s == k:
            remove_attributes.append(a)

    reduct = list(set(attributes) - set(remove_attributes))
    reduct.sort()
    return reduct, k, remove_attributes


from models.roughsets import QuickReduct, RoughSets 

class VariablePrecisionRoughSet(RoughSets): 
  def __init__(self, df, beta):
    self.beta = beta;
    self.dataset = df
    

  def lower_approximation(self, parameter, keputusan):
    lower_approximation = dict()
    X = self.kelompokan_berdasarkan_keputusan(keputusan)
    ind = self.kelompokan_berdasarkan_parameter(parameter)
    for key, concept in X.items():
      lower_approximation[key] = []
      for g in ind:
        intr = set(concept).intersection(set(g))
        e = 1 - len(intr)/len(g)
        if e <= self.beta: 
          lower_approximation[key] += g

    return lower_approximation

class QuickReductVPRS(QuickReduct):
  def __init__(self, rs):
    self.model = rs 

  def reduct(self, attributes, decision):
    k_c = self.model.konsistensi_tabel(attributes, decision)

    R = []
    while True:
      T = R 
      S = set(attributes) - set(R)
      for x in list(S):
        k_r_plus_x = self.model.konsistensi_tabel(R+[x], decision)
        k_t = self.model.konsistensi_tabel(T, decision)

        if k_r_plus_x > k_t:
          T = R+[x]
      R = T

      k_r = self.model.konsistensi_tabel(R, decision)

      if k_r == k_c: 
        break

    return R 


class MDA():
    def __init__(self, df, attrs):
        self.dataset = df 
        self.attributes = attrs

    def __group(self, attr):
        g = dict()

        for index, row in self.dataset[[attr]].iterrows():
            if row[attr] not in g:
                g[row[attr]] = list() 
            g[row[attr]].append(index)
        
        return g
    
    def group(self):
        ind = dict()
        for attr in self.attributes:
            ind[attr] = self.__group(attr)

        return ind

    def calculate_dependency_attributes(self):
        m = dict()
        for val, glumps in self.group().items():
            m[val] = list()
            for val2, glumps2 in self.group().items():
                if val2 == val:
                    continue
                # m[val][val2] = 0

                # print(val2, val)
                ns = 0
                for ka, a in glumps.items():
                    set_a = set(a)
                    for kb, b in glumps2.items():
                        set_b = set(b)
                        if set_b.issubset(set_a):
                            ns = ns + len(set_b)
                m[val].append(ns/len(self.dataset))
            m[val].sort(reverse=True)
        
        return m
    
    def sort_attributes_by_dependency(self):
        remove = list()
        # identic = list()
        for attr, val in self.calculate_dependency_attributes().items():
            if attr in remove:
                continue

            for attr2, val2 in self.calculate_dependency_attributes().items():
                if attr2 in remove:
                    continue
                if attr == attr2:
                    continue
                i = 0 

                is_remove = False
                if val[i] == val2[i]:
                    is_remove= True

                while True:
                    if val[i] != val2[i]:
                        break; 

                    if i >= len(val)-1:
                        break
                    i = i + 1
                
                if is_remove == True:
                    if val[i] < val2[i]:
                        remove.append(attr)
                    else :
                        remove.append(attr2)
                        # if val[i] == val2[i]:
                        #     identic.append(attr)
        
        # print(identic)
        ''' Creating new list of attributes without the redundant one '''
        attr_order = dict()
        for attr, val in self.calculate_dependency_attributes().items():
            if attr not in remove:
                attr_order[attr] = val[0]

        ''' Mengurutkan dictionary '''
        return sorted(attr_order.items(), key=lambda x: x[1], reverse=True)    
    
    def run(self):
        return self.sort_attributes_by_dependency(), self.calculate_dependency_attributes()


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rst_tools', 'rst_tools.models']

package_data = \
{'': ['*']}

install_requires = \
['pandas']

setup_kwargs = {
    'name': 'rst-tools',
    'version': '0.1.7',
    'description': 'Rough sets implementation for Python',
    'long_description': '# Guidelines \n\n## Instalation\nTo install this package use pip as following \n\n`pip install rst-tools`\n\n## Usage \n\n### Rough Set (Pawlak\'s Model)\n\n```python\nimport pandas as pd\nfrom rst_tools.models.roughsets import RoughSets as RST\nfrom rst_tools.models.roughsets import QuickReduct as QR\n\ndf = pd.read_csv("YOUR/FILE.csv")\nconditional_attributes = list(df.column.values)\ndecision_attribute = conditional_attributes[-1]\ndel conditional_attributes[-1]\n\nroughsets = RST(df)\nkonsistensi_data = roughsets.konsistensi_tabel(conditional_attributes, decision_attribute)\n\nprint("Konsistensi Data %f" % (konsistensi_data))\n\nqr = QR(roughsets)\nreducts = qr.reduct(decision_attributes, decision_attribute) \n\nprint("Hasil reduksi atribut %s" % (reducts))\n\n\'\'\'....\'\'\'\n```\n\n### Maximum Dependency Attribute (MDA) (Herawan\'s Model)\n\n```python\nimport pandas as pd\nfrom rst_tools.models.maximum_dependency_attributes import MDA\n\n\ndf = pd.read_csv("YOUR/FILE.csv")\nconditional_attributes = list(df.column.values)\ndecision_attribute = conditional_attributes[-1]\ndel conditional_attributes[-1]\n\nmda = MDA(df, attrs=attributes)\nattributes, table = mda.run()\n\nprint(attributes) \n\n\'\'\'....\'\'\'\n```\n\n### Variable Precision Rough Sets (Ziarko\'s Model)\n\n```python\nimport pandas as pd\nfrom rst_tools.models.vprs import VariablePrecisionRoughSet\nfrom rst_tools.models.reduct import reduct\n\n\ndf = pd.read_csv("YOUR/FILE.csv")\nconditional_attributes = list(df.column.values)\ndecision_attribute = conditional_attributes[-1]\ndel conditional_attributes[-1]\n\nrs = vprs(df, b) \nreducts, k, sup_attrs = reduct(rs, attributes, decision)\n\nprint("Konsistensi Data %f" % (k))\nprint("Atribut reduksi %s" % (reducts))\n\n\'\'\'....\'\'\'\n```',
    'author': 'Rofilde Hasudungan',
    'author_email': 'rofilde@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/rhsitorus/rst-tools',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
